<?php
/**
 * The middlware debug output system. A *real* or concrete output class shall
 * become registered with this middleware because the back-fall class will
 * become deprecated soon.
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2012 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class DebugMiddleware extends BaseMiddleware implements Registerable {
	/**
	 * The concrete output instance
	 */
	private $outputInstance = NULL;

	/**
	 * An instance of this class
	 */
	private static $selfInstance = NULL;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set own instance
		self::$selfInstance = $this;

		// Set it so all can use it
		$this->setDebugInstance($this);
	}

	/**
	 * Create a new debug output system.
	 * If no output is given this class is currently being used for back-fall.
	 * This fall-back mechanism will become deprecated very soon.
	 *
	 * @param	$debuggerClass	The class name which we shall use for
	 *							registering the *real* debug output
	 * @return	$debugInstance	An instance of this middleware class
	 */
	public static final function createDebugMiddleware ($debuggerClass) {
		// Create an instance if this middleware
		$debugInstance = new DebugMiddleware();

		// Is there a valid output instance provided?
		if ((!is_null($debuggerClass)) && (is_object($debuggerClass)) && ($debuggerClass instanceof OutputStreamer)) {
			// Use the given output instance
			$debugInstance->setOutputInstance($debuggerClass);
		} elseif ((!is_null($debuggerClass)) && (is_string($debuggerClass)) && (class_exists($debuggerClass))) {
			// A name for a debug output class has been provided so we try to get it
			$debuggerInstance = ObjectFactory::createObjectByName($debuggerClass);

			// Set this as output class
			$debugInstance->setOutputInstance($debuggerInstance);
		}

		// Return instance
		return $debugInstance;
	}

	/**
	 * Getter for an instance of this class
	 *
	 * @return	$selfInstance		An instance of this class
	 */
	public static final function getSelfInstance() {
		return self::$selfInstance;
	}

	/**
	 * Setter for output instance
	 *
	 * @param	$outputInstance	The debug output instance
	 * @return	void
	 */
	public final function setOutputInstance (OutputStreamer $outputInstance) {
		$this->outputInstance = $outputInstance;
	}

	/**
	 * This method shall send debug output which can be HTML code for the
	 * browser or debug lines for a log file, etc. to the registered debug
	 * output instance.
	 *
	 * @param	$outStream	Data we shall 'stream' out to the world
	 * @param	$stripTags	Whether HTML tags shall be stripped out
	 * @return	void
	 */
	public final function output ($outStream, $stripTags = FALSE) {
		// Is the output stream set
		if (empty($outStream)) {
			// @TODO Initialization phase
			return;
		} // END - if

		// Use the output instance
		$this->outputInstance->outputStream($outStream, $stripTags);
	}
}

// [EOF]
?>
